@@ -1,6 +1,5 @@
 library(lme4)
 lme(mAch ~ meanses*cses + sector*cses,
     data = Hsb82, random = ~ cses|school)
-show(lme1(mAch ~ meanses*cses + sector*cses,
-          data = Hsb82, random = ~ cses|school))
+lmer(mAch ~ meanses*cses + sector*cses + (cses|school), Hsb82)
 q("no")
